/*
        Copyright (c) 1998, Apple Computer, Inc.
        All rights reserved.

        You may freely copy, distribute and reuse the code in this example.
        Apple disclaims any warranty of any kind, expressed or implied,
        as to its fitness for any particular use.
*/

#import "FlatFileColumn.h"
#import "FlatFileContext.h"
#import <EOAccess/EOAccess.h>

#define DataBegin '<'
#define DataEnd   '>'

@implementation FlatFileColumn

+ (Class)columnClassForAttribute:(EOAttribute *)attr
{
    switch ([attr adaptorValueType]) {
        case EOAdaptorNumberType : return [FlatFileNumberColumn class];
        case EOAdaptorCharactersType : return [FlatFileStringColumn class];
        case EOAdaptorBytesType : return [FlatFileDataColumn class];
        case EOAdaptorDateType : return [FlatFileDateColumn class];
    }

    return nil;
}

+ columnForAttribute:(EOAttribute *)attr channel:(FlatFileChannel *)channel
{
    return [[[[self columnClassForAttribute:attr] alloc] initWithAttribute:attr channel:channel] autorelease];
}

- initWithAttribute:(EOAttribute *)attr channel:(FlatFileChannel *)channel
{
    [super init];
    attribute = [attr retain];

    return self;
}

- (EOAttribute *)attribute
{
    return attribute;
}

- (void)dealloc
{
    [attribute release];
    [super dealloc];
}

- (void)bindAtIndex:(int)index
{
    bindingIndex = index;
}

- (int)bindingIndex
{
    return bindingIndex;
}

- (id)newValueForBytes:(void *)bytes length:(int)length
{
    return nil;
}

@end

@implementation FlatFileNumberColumn

- initWithAttribute:(EOAttribute *)attr channel:(FlatFileChannel *)channel
{
    [super initWithAttribute:attr channel:channel];

    if ([[attribute valueClassName] isEqualToString:@"NSDecimalNumber"])
        valueType = EODecimalNumberType;
    else if ([[attribute valueType] isEqualToString:@"i"])
        valueType = EOIntegerType;
    else
        valueType = EODoubleType;

    return self;
}

- (id)newValueForBytes:(void *)bytes length:(int)length
{
    NSString *helper = [[NSString alloc] initWithCString:(char *)bytes length:length];
    id value;
    
    switch (valueType) {
        case EOIntegerType :
            value = [[NSNumber alloc] initWithInt:[helper intValue]];
            break;
        case EODoubleType :
            value = [[NSNumber alloc] initWithDouble:[helper doubleValue]];
            break;
        default :
            value = [[NSDecimalNumber decimalNumberWithString:helper] retain];
    }            

    [helper release];
    return value;
}

@end

@implementation FlatFileStringColumn

- initWithAttribute:(EOAttribute *)attr channel:(FlatFileChannel *)channel
{
    return [self initWithAttribute:attr channel:channel encoding:[(FlatFileContext *)[channel adaptorContext] databaseEncoding]];
}

- initWithAttribute:(EOAttribute *)attr channel:(FlatFileChannel *)channel encoding:(NSStringEncoding)anEncoding
{
    [super initWithAttribute:attr channel:channel];
    [self setEncoding:anEncoding];

    return self;
}

- (NSStringEncoding)encoding
{
    return encoding;
}

- (void)setEncoding:(NSStringEncoding)anEncoding
{
    encoding = anEncoding;
}

- (id)newValueForBytes:(void *)bytes length:(int)length
{
    return [attribute newValueForBytes:bytes length:length encoding:encoding];
}

@end

@implementation FlatFileDataColumn

- (id)newValueForBytes:(void *)bytes length:(int)length
{
    char *valueBytes = (char *)malloc(length * sizeof(char)), *currentValueByte = valueBytes, byte = 0, nibble;
    int i = 0, valueLength = 0;
    id value;
    BOOL haveFirstNibble = NO;
    
    if (*(char *)bytes == DataBegin)
        bytes++;

    while ((i++ < length) && ((nibble = *(char *)bytes++) != DataEnd)) {
        if ((nibble >= '0') && (nibble <= '9'))
            nibble -= '0';
        else if ((nibble >= 'a') && (nibble <= 'z'))
            nibble -= ('a' - 10);
        else
            continue;

        if (haveFirstNibble) {
            byte += nibble;
            *currentValueByte++ = byte;
            valueLength++;
        } else
            byte = nibble << 4;

        haveFirstNibble = !haveFirstNibble;
    }

    value = [attribute newValueForBytes:valueBytes length:valueLength];
    free(valueBytes);
    return value;
}

@end

@implementation FlatFileDateColumn

- (id)newValueForBytes:(void *)bytes length:(int)length
{
    NSString *helper = [NSString stringWithCString:bytes length:length];
    return [[NSCalendarDate dateWithString:helper calendarFormat:FlatFileCalendarFormat] retain];
}

@end
